@@ -92,6 +92,7 @@ static gboolean handle_exposed_banner(GtkWidget * widget,
                                       struct frontend * fe)
 {
     struct frontend_data * fe_data = fe->data;
+    GdkScreen * screen;
     GdkWindow * window;
     PangoFontDescription * font;
     PangoLayout * layout;
@@ -134,12 +135,13 @@ static gboolean handle_exposed_banner(GtkWidget * widget,
         font = pango_font_description_from_string("Sans 12");
         pango_layout_set_font_description(layout, font);
         pango_layout_get_pixel_size(layout, &text_width, &text_height);
+        screen = gtk_window_get_screen(GTK_WINDOW(fe_data->window));
         window = gtk_widget_get_window(widget);
-        /* Left-align, vertically-center */
+        /* Right-align, vertically-center */
         cairo_t *cr = gdk_cairo_create(window);
         cairo_move_to(cr,
-                        DEFAULT_PADDING * 2,
-                        (fe_data->logo_height - text_height) / 2);
+                      gdk_screen_get_width(screen) - text_width - DEFAULT_PADDING * 2,
+                      (fe_data->logo_height - text_height) / 2);
         pango_cairo_show_layout(cr, layout);
         cairo_destroy(cr);
 
@@ -202,7 +204,6 @@ static void create_banner(struct frontend * fe, GtkWidget * container)
     }
     g_free(theme_name);
 
-    /* XXX: check NULL! */
     banner = gtk_event_box_new();
     logo = gtk_image_new_from_file(banner_path);
     gtk_misc_set_alignment(GTK_MISC(logo), 0.5 /* center */, 0 /* top */);
@@ -234,6 +235,10 @@ static void create_banner(struct frontend * fe, GtkWidget * container)
     gtk_box_pack_start(GTK_BOX(container), banner,
                        FALSE /* don't expand */, FALSE /* don't fill */,
                        0 /* padding */);
+
+    /* Workaround for #882804, GTK 2 only */
+    fe_data->banner_widget = banner;
+    fe_data->banner_workaround_needed = TRUE;
 }
 
 /** Create the label where the current operation will be displayed.
@@ -246,7 +251,6 @@ static void create_label_title(struct frontend * fe, GtkWidget * container)
     struct frontend_data * fe_data = fe->data;
     GtkWidget * label_title;
 
-    /* check NULL! */
     label_title = gtk_label_new(NULL /* no label */);
 
     gtk_misc_set_alignment(GTK_MISC(label_title), 0 /* left */, 0 /* top */);
@@ -294,7 +298,6 @@ static void create_action_box(struct frontend * fe, GtkWidget * container)
 
     g_assert(NULL == fe_data->action_box);
 
-    /* check NULL! */
     action_box = gtk_hbutton_box_new();
 
     gtk_button_box_set_layout(GTK_BUTTON_BOX(action_box), GTK_BUTTONBOX_END);
@@ -322,14 +325,11 @@ static void create_main_widgets(struct frontend * fe, GtkWidget * window)
     GtkWidget * v_mainbox;
     GtkWidget * h_mainbox;
 
-    /* check NULL! */
     outer_box = gtk_vbox_new(FALSE /* don't make children equal */,
                              0 /* padding */);
     create_banner(fe, outer_box);
-    /* check NULL! */
     v_mainbox = gtk_vbox_new(FALSE /* don't make children equal */,
                              0 /* padding */);
-    /* check NULL! */
     h_mainbox = gtk_hbox_new(FALSE /* don't make children equal */,
                              0 /* padding */);
     create_label_title(fe, v_mainbox);
@@ -432,7 +432,6 @@ void cdebconf_gtk_add_global_key_handler(struct frontend * fe,
     struct frontend_data * fe_data = fe->data;
     struct shortcut * shortcut;
 
-    /* XXX: check NULL! */
     shortcut = g_malloc0(sizeof (struct shortcut));
     shortcut->window = fe_data->window;
     shortcut->handler_id = g_signal_connect_after(
@@ -466,10 +465,8 @@ void cdebconf_gtk_center_widget(GtkWidget ** widget, guint horizontal_padding,
     GtkWidget * vbox;
     GtkWidget * hbox;
 
-    /* XXX: check NULL! */
     vbox = gtk_vbox_new(FALSE /* don't make children equal */,
                         0 /* padding */);
-    /* XXX: check NULL! */
     hbox = gtk_hbox_new(FALSE /* don't make children equal */,
                         0 /* padding */);
     gtk_box_pack_start(GTK_BOX(vbox), *widget, TRUE /* expand */,
@@ -492,12 +489,10 @@ static GtkWidget * create_dialog_action_box(struct frontend * fe,
     GtkWidget * close_button;
     char * label;
 
-    /* check NULL! */
     action_box = gtk_hbutton_box_new();
     gtk_button_box_set_layout(GTK_BUTTON_BOX(action_box), GTK_BUTTONBOX_END);
 
     label = cdebconf_gtk_get_text(fe, "debconf/button-continue", "Continue");
-    /* check NULL! */
     close_button = gtk_button_new_with_label(label);
     g_free(label);
 
@@ -519,7 +514,6 @@ static GtkWidget * create_dialog_title_label(const gchar * title)
     GtkWidget * label;
     gchar * markup;
 
-    /* check NULL! */
     label = gtk_label_new(NULL /* no text */);
     gtk_misc_set_alignment(GTK_MISC(label), 0 /* left aligned */,
                            0 /* top aligned */);
@@ -542,7 +536,6 @@ gboolean cdebconf_gtk_run_message_dialog(struct frontend * fe,
     GtkWidget * frame;
     GtkWidget * label;
 
-    /* XXX: check NULL! */
     dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_modal(GTK_WINDOW(dialog), TRUE /* modal */);
     gtk_window_set_transient_for(GTK_WINDOW(dialog),
@@ -556,7 +549,6 @@ gboolean cdebconf_gtk_run_message_dialog(struct frontend * fe,
     label = gtk_label_new(message);
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 
-    /* XXX: check NULL! */
     vbox = gtk_vbox_new(FALSE /* don't make children equal */,
                         DEFAULT_PADDING);
     gtk_box_pack_start(GTK_BOX(vbox), create_dialog_title_label(title),
@@ -681,7 +673,7 @@ void cdebconf_gtk_empty_target_box(struct frontend * fe)
     struct frontend_data * fe_data = fe->data;
 
     gtk_container_forall(GTK_CONTAINER(fe_data->target_box),
-                         (GtkCallback) gtk_widget_destroy, NULL /* no data */);
+                         cdebconf_gtk_callback_widget_destroy, NULL /* no data */);
 }
 
 /** Returns the width (in pango units) of the given text.
@@ -702,5 +694,11 @@ gint cdebconf_gtk_get_text_width(GtkWidget * widget, gchar * text)
     return width;
 }
 
+/** GtkCallback wrapper for gtk_widget_destroy */
+void cdebconf_gtk_callback_widget_destroy(GtkWidget *widget, gpointer data)
+{
+    gtk_widget_destroy(widget);
+}
+
 /* vim: et sw=4 si
  */
