@@ -74,6 +74,16 @@ typedef int (text_handler)(struct frontend *obj, unsigned printed, struct questi
 #define ISEMPTY(buf) (buf[0] == 0)
 #endif
 
+#define PROGRESS_NONE -10
+static int last_progress = PROGRESS_NONE;
+
+static sigset_t sigint_set;
+
+static const struct timespec null_timeout = {
+	.tv_sec = 0,
+	.tv_nsec = 0
+};
+
 /*
  * Function: getwidth
  * Input: none
@@ -975,9 +985,6 @@ static struct question_handlers {
  * Output: int - DC_OK, DC_NOTOK
  * Description: initializes the text UI
  * Assumptions: none
- *
- * TODO: SIGINT is ignored by the text UI, otherwise it interfers with
- * the way question/answers are handled. this is probably not optimal
  */
 static int text_initialize(struct frontend *obj, struct configuration *conf)
 {
@@ -987,7 +994,13 @@ static int text_initialize(struct frontend *obj, struct configuration *conf)
 	data->previous_title = NULL;
 	obj->data = data;
 	obj->interactive = 1;
+
+	/* Prepare for catching SIGINT synchronously */
 	signal(SIGINT, SIG_IGN);
+	sigemptyset(&sigint_set);
+	sigaddset(&sigint_set, SIGINT);
+	sigprocmask(SIG_BLOCK, &sigint_set, NULL);
+
 	if (palette && !strcmp(palette, "dark") &&
 			term && (!strcmp(term, "linux") || !strcmp(term, "bterm"))) {
 		/* Hard-code for these cases */
@@ -1024,6 +1037,19 @@ text_can_go_back(struct frontend *obj, struct question *q)
 	return (obj->capability & DCF_CAPB_BACKUP);
 }
 
+/*
+ * Function: text_can_cancel_progress
+ * Input: struct frontend *obj - frontend object
+ * Output: boolean
+ * Description: tells whether confmodule supports cancelling a step
+ * Assumptions: none
+ */
+static bool
+text_can_cancel_progress(struct frontend *obj)
+{
+    return (obj->capability & DCF_CAPB_PROGRESSCANCEL);
+}
+
 /*
  * Function: text_can_align
  * Input: struct frontend *obj - frontend object
@@ -1078,6 +1104,10 @@ static int text_go(struct frontend *obj)
 	int ret = DC_OK;
 	unsigned printed;
 
+	/* Make sure that after asking the questions we show that we are back to
+	 * progressing. */
+	last_progress = PROGRESS_NONE;
+
 	while (q != NULL) {
 		for (i = 0; i < DIM(question_handlers); i++) {
 			text_handler *handler;
@@ -1162,22 +1192,29 @@ static void text_progress_start(struct frontend *obj, int min, int max, struct q
 	printf("%s  ", title_desc);
 	free(title_desc);
 	fflush(stdout);
+
+	/* Clear any previous SIGINT */
+	int res = sigtimedwait(&sigint_set, NULL, &null_timeout);
 }
 
 static int text_progress_set(struct frontend *obj, int val)
 {
-	static int last = 0;
 	int new;
 
+	/* Catch SIGINT, and in that case, cancel the current step */
+	if (obj->methods.can_cancel_progress(obj)
+		&& sigtimedwait(&sigint_set, NULL, &null_timeout) == SIGINT)
+		return DC_GOBACK;
+
 	obj->progress_cur = val;
 	new = ((double)(obj->progress_cur - obj->progress_min) / 
 		(double)(obj->progress_max - obj->progress_min) * 100.0);
-	if (new < last)
-		last = 0;
+	if (new < last_progress)
+		last_progress = PROGRESS_NONE;
 	/*  Prevent verbose output  */
-	if (new / 10 == last / 10)
+	if (new / 10 == last_progress / 10)
 		return DC_OK;
-	last = new;
+	last_progress = new;
 	printf("... %d%%", new);
 	fflush(stdout);
 
@@ -1199,6 +1236,7 @@ struct frontend_module debconf_frontend_module =
 	.shutdown = text_shutdown,
 	.lookup_directive = text_lookup_directive,
 	.can_go_back = text_can_go_back,
+	.can_cancel_progress = text_can_cancel_progress,
 	.can_align = text_can_align,
 	.go = text_go,
 	.progress_start = text_progress_start,
